import { Hono } from "hono";
import { cors } from "hono/cors";
const kv = await Deno.openKv();
export function routeHome(config, ctx) {
    return ctx.json({
        system: config.options.name,
        motd: config.options.messages.welcome,
        items: config.options.items,
        commands: [
            "add",
            "clear",
        ],
    });
}
export async function routeItem(item, ctx) {
    var _a;
    const itemKey = ["items", item];
    const getResult = await kv.get(itemKey);
    const records = (_a = getResult === null || getResult === void 0 ? void 0 : getResult.value) !== null && _a !== void 0 ? _a : [];
    return ctx.json(records);
}
export async function routeItemsAdd(item, ctx) {
    var _a;
    const itemKey = ["items", item];
    const getResult = await kv.get(itemKey);
    const records = (_a = getResult === null || getResult === void 0 ? void 0 : getResult.value) !== null && _a !== void 0 ? _a : [];
    const value = ctx.req.param("value");
    records.push(value);
    await kv.set(itemKey, records);
    return ctx.redirect(`/${item}`);
}
export async function routeItemsClear(item, ctx) {
    const itemKey = ["items", item];
    await kv.set(itemKey, []);
    return ctx.redirect(`/${item}`);
}
export function validateConfig(options) {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    return {
        options: {
            name: (_a = options === null || options === void 0 ? void 0 : options.isim) !== null && _a !== void 0 ? _a : "hizli-api",
            messages: {
                welcome: (_c = (_b = options === null || options === void 0 ? void 0 : options.mesajlar) === null || _b === void 0 ? void 0 : _b.hosgeldin) !== null && _c !== void 0 ? _c : "Hoş geldin!",
                success: (_e = (_d = options === null || options === void 0 ? void 0 : options.mesajlar) === null || _d === void 0 ? void 0 : _d.basarili) !== null && _e !== void 0 ? _e : "Başarılı",
                fail: (_g = (_f = options === null || options === void 0 ? void 0 : options.mesajlar) === null || _f === void 0 ? void 0 : _f.basarisiz) !== null && _g !== void 0 ? _g : "Başarısız",
            },
            items: (_h = options === null || options === void 0 ? void 0 : options.nesneler) !== null && _h !== void 0 ? _h : [],
        },
    };
}
export function hizliApi(options) {
    const config = validateConfig(options);
    const app = new Hono();
    app.use("*", cors({
        origin: "*",
        allowMethods: ["GET", "OPTIONS"],
        allowHeaders: ["Content-Type"],
    }));
    app.get("/", (ctx) => routeHome(config, ctx));
    for (const item of config.options.items) {
        app.get(`/${item}`, (ctx) => routeItem(item, ctx));
        app.get(`/${item}/add/:value`, (ctx) => routeItemsAdd(item, ctx));
        app.get(`/${item}/clear`, (ctx) => routeItemsClear(item, ctx));
    }
    Deno.serve(app.fetch);
}
export { hizliApi as default };
