import { Hono } from "hono";
import { cors } from "hono/cors";

export interface Options {
  isim?: string;
  mesajlar?: {
    hosgeldin?: string;
    basarili?: string;
    basarisiz?: string;
  };
  nesneler?: string[];
}

export interface Config {
  options: {
    name: string;
    messages: {
      welcome: string;
      success: string;
      fail: string;
    };
    items: string[];
  };
}

// @ts-ignore Deno namespace
const kv = await Deno.openKv();

// deno-lint-ignore no-explicit-any
export function routeHome(config: Config, ctx: any) {
  return ctx.json({
    system: config.options.name,
    motd: config.options.messages.welcome,
    items: config.options.items,
    commands: [
      "add",
      "clear",
    ],
  });
}

// deno-lint-ignore no-explicit-any
export async function routeItem(item: any, ctx: any) {
  const itemKey = ["items", item];

  const getResult = await kv.get(itemKey);
  const records = getResult?.value ?? [];

  return ctx.json(records);
}

// deno-lint-ignore no-explicit-any
export async function routeItemsAdd(item: any, ctx: any) {
  const itemKey = ["items", item];

  const getResult = await kv.get(itemKey);
  // deno-lint-ignore no-explicit-any
  const records: any = getResult?.value ?? [];

  const value = ctx.req.param("value");
  records.push(value);

  await kv.set(itemKey, records);

  return ctx.redirect(`/${item}`);
}

// deno-lint-ignore no-explicit-any
export async function routeItemsClear(item: any, ctx: any) {
  const itemKey = ["items", item];

  await kv.set(itemKey, []);

  return ctx.redirect(`/${item}`);
}

export function validateConfig(options: Options) {
  return {
    options: {
      name: options?.isim ?? "hizli-api",
      messages: {
        welcome: options?.mesajlar?.hosgeldin ?? "Hoş geldin!",
        success: options?.mesajlar?.basarili ?? "Başarılı",
        fail: options?.mesajlar?.basarisiz ?? "Başarısız",
      },

      items: options?.nesneler ?? [],
    },
  };
}

export function hizliApi(options: Options) {
  const config = validateConfig(options);

  const app = new Hono();
  app.use("*", cors({
    origin: "*",
    allowMethods: ["GET", "OPTIONS"],
    allowHeaders: ["Content-Type"],
  }));

  app.get("/", (ctx) => routeHome(config, ctx));
  for (const item of config.options.items) {
    app.get(`/${item}`, (ctx) => routeItem(item, ctx));
    app.get(`/${item}/add/:value`, (ctx) => routeItemsAdd(item, ctx));
    app.get(`/${item}/clear`, (ctx) => routeItemsClear(item, ctx));
  }

  // @ts-ignore Deno namespace
  Deno.serve(app.fetch);
}

export { hizliApi as default };
